const { imageClear } = require("../utils/logic/helper");
const asyncHandler = require("express-async-handler");
const BannerModel = require("../models/BannerModel");
const { uploadMultipleFile } = require("../Middlewares/uploadImageMiddleware");
const BannerResource = require("../utils/resources/BannerResource");

exports.UploadImage = uploadMultipleFile(
  ["image_ar", "image_en"],
  "images/banner"
);

exports.Add = async (req, res) => {
  try {
    const { title, default_link } = req.body;
    const image_ar = req.image_ar;
    const image_en = req.image_en;
    // Debugging logs to ensure correct data is being received
    console.log("Request Body:", req.body);
    // Create new banner
    const newBanner = await BannerModel.create({
      title,
      default_link,
      image_ar,
      image_en,
    });

    res.status(200).json({
      success: true,
      Banner: newBanner,
    });
  } catch (error) {
    console.error("Error adding Banner:", error);
    res.status(500).json({ error: error.message });
  }
};

exports.GetBanner = asyncHandler(async (req, res) => {
  try {
    const lang = req.headers["accept-language"] || "en";

    let Banners = await BannerModel.find({});

    const data = await BannerResource.collection(Banners, lang);

    res.status(200).json(data);
  } catch (error) {
    res.status(500).json({
      success: false,
      message: "Server Error",
      error: error.message,
    });
  }
});

exports.DeleteBanner = asyncHandler(async (req, res, next) => {
  const { id } = req.params;
  const Banner = await BannerModel.findById(id);

  imageClear(Banner.image_ar);
  imageClear(Banner.image_en);

  if (!Banner) {
    const error = new Error("Banner not founded ");
    error.statusCode = 422;
    throw error;
  }
  await Banner.deleteOne();
  res.status(200).json({ msg: "Banner deleted" });
});

exports.UpdateBanner = asyncHandler(async (req, res, next) => {
  const updateData = {};
  const { id } = req.params;
  const { title, default_link } = req.body;
  if (req.image_ar) {
    const image_ar = req.image_ar;
    updateData.image_ar = image_ar;
  }
  if (req.image_en) {
    const image_en = req.image_en;
    updateData.image_en = image_en;
  }
  if (title) {
    console.log('tttttttttt');
    updateData.title = title;
  }
  if (default_link) {
    updateData.default_link = default_link;
  }
console.log('updatedData',updateData);
  const oldBanner = await BannerModel.findOne({ _id: id });

  const banner = await BannerModel.findOneAndUpdate(
    { _id: id },
    updateData,
    { new: true }
  );

  if (
    updateData.image !== undefined &&
    updateData.image !== oldBanner.image
  ) {
    imageClear(oldBanner.image);
  }
  if (!banner) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  res.status(200).json({ data: banner });
});
