const asyncHandler = require("express-async-handler");
const UserModel = require("../models/UserModel");
const ApiError = require("../utils/apiError");
const { uploadSingleImage } = require("../Middlewares/uploadImageMiddleware");
const { imageClear } = require("../utils/logic/helper");
const { send, Validate } = require("../utils/logic/otp");
const { extractPhoneDetails } = require("../utils/logic/helper");

exports.UploadImage = uploadSingleImage("image", "images/user");

exports.UpdateUser = asyncHandler(async (req, res, next) => {
  try {
    const updateData = {};
    const id = req.user._id;
    const { name, fcm_token, language } = req.body;

    if (req.file) {
      const image = req.file.path;
      updateData.image = image;
    }
    if (name) {
      updateData.name = name;
    }
    if (fcm_token) {
      console.log("update_fcm_token:", fcm_token);
      updateData.fcm_token = fcm_token;
    }
    if (language) {
      console.log("language:", language);
      updateData.language = language;
    }
    const oldUser = await UserModel.findById(id);

    if (!oldUser) {
      return next(new ApiError(`User not found`, 404));
    }

    const user = await UserModel.findByIdAndUpdate(id, updateData, {
      new: true,
    });

    if (!user) {
      return next(new ApiError(`Failed to update user`, 400));
    }

    if (
      updateData.image &&
      oldUser.image &&
      updateData.image !== oldUser.image
    ) {
      await imageClear(oldUser.image, "user");
    }

    res.status(200).json({ data: user });
  } catch (error) {
    next(new ApiError(error.message, 500));
  }
});

exports.DeleteUser = asyncHandler(async (req, res, next) => {
  const id = req.user._id;
  console.log("helllllllllllllllllllllllllo");
  const user = await UserModel.findByIdAndUpdate(
    id,
    { status: false },
    { new: true }
  );
  if (!user) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  res.status(200).json({ msg: "User deleted (status set to false)" });
});

exports.GetUser = asyncHandler(async (req, res, next) => {
  const id = req.user._id;
  const user = await UserModel.findById(id);
  if (!user) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  res.status(200).json({ user });
});

exports.sendOtp = async (req, res) => {
  console.log("Full body", req.body);
  const phone = req.body.phone;
  const existingUser = await UserModel.findOne({ phone });

  if (!existingUser) {
    return next(new ApiError("User does not exist", 404));
  }
  if (existingUser.status === false) {
    return res.status(403).json({
      success: false,
      message:
        "Your account is already deactivated. Please contact support for help ",
    });
  }
  let countryCode, mobileNumber;
  try {
    ({ countryCode, mobileNumber } = extractPhoneDetails(phone));
  } catch (error) {
    console.error(error.message);
  }
  try {
    const otp = await send(countryCode, mobileNumber);
    const verificationId = otp.data.verificationId;

    res.render("delete_account_otp", {
      otpSent: true,
      verified: false,
      phone,
      verificationId,
    });
  } catch (error) {
    console.error("❌ Error sending OTP:", error.message);

    // Wait for 2 seconds then redirect back or show error
    setTimeout(() => {
      res.render("delete_account_otp", {
        otpSent: false,
        verified: false,
        phone: null,
        errorMessage: "Failed to send OTP. Please try again.",
      });
    }, 2000);
  }
};
exports.verifyOtp = async (req, res, next) => {
  console.log("Full body:", req.body);
  const { verificationId, phone, otp } = req.body;

  let countryCode, mobileNumber;

  try {
    // Example function to extract country code and number
    ({ countryCode, mobileNumber } = extractPhoneDetails(phone));
  } catch (error) {
    return next(new ApiError("Invalid phone format", 400));
  }

  try {
    const existingUser = await UserModel.findOne({ phone });

    if (!existingUser) {
      return next(new ApiError("User does not exist", 404));
    }

    if (existingUser.role !== "google_play") {
      const validateOtpResponse = await Validate(
        verificationId,
        countryCode,
        mobileNumber,
        otp // ❗ make sure you pass `otp`, not `code`
      );

      console.log("validateOtpResponse:", validateOtpResponse);

      if (
        validateOtpResponse.data.verificationStatus !== "VERIFICATION_COMPLETED"
      ) {
        return next(new ApiError("Invalid OTP code", 400));
      }
    }

    // ❗ Use `{ phone: phone }` as the filter in findOneAndUpdate
    await UserModel.findOneAndUpdate(
      { phone: phone },
      { status: false },
      { new: true }
    );

    // ✅ OTP Verified and account disabled
    res.render("delete_account_otp", {
      otpSent: true,
      verified: true,
      phone,
    });
  } catch (error) {
    console.log("❌ Error verifying OTP:", error);
    next(new ApiError("Something went wrong", 500));
  }
};
