// sockets/index.js
const UserModel = require("./models/UserModel");
const ServiceModel = require("./models/ServiceModel");
const {
  NewServiceSocket,
  GetServicesOfUserocket,
  GetDetails,
  DeleteService,
} = require("./Controllers/ServiceController");
const {
  check_conversation,
  send_message,
  GetMessage,
  getHomeConversations,
} = require("./utils/logic/chat");
const {
  GetMyOfferSocket,
  UpdateOfferSocket,
  UpdateStatusOfferSocket,
  AddOfferSocket,
  StatusOfferSocket,
} = require("./Controllers/OfferController");
const { send_notification } = require("./Controllers/NotificationController");

module.exports = (io, app) => {
  const userSockets = new Map();
  const userSocketServices = new Map();
  // Chat namespace
  const chat = io.of("/SocketChat");
  chat.on("connection", (socket) => {
    console.log("🔵 New client connected:", socket.id);

    socket.on("chat", async (data) => {
      const {
        userId,
        receiver,
        service,
        offer,
        content: inputContent,
        event,
        price,
        type,
        DropReasons,
        conversation_id,
      } = data;
      let content = inputContent;
      console.log("🔔 [SocketService] Connected:", socket.id);
      if (userId) {
        console.log("hiiiiiiii22ii");
        userSocketServices.set(userId, socket.id);
        socket.join(`user_${userId}`);
        console.log(
          `✅ User ${userId} registered with SocketService ${socket.id} and joined roomm user_${userId}`
        );
      }
      try {
        const senderSocket = chat.sockets.get(socket.id);
        if (event === "send") {
          let messageType = "normal";
          const user_receiver = await UserModel.findById(receiver);
          const user_sender = await UserModel.findById(userId);
          if (typeof type !== "undefined") {
            if (type === "update-offer") {
              if (typeof price !== "undefined") {
                await UpdateOfferSocket(offer, price);
                content =
                  user_receiver?.language === "en"
                    ? `A new offer has been submitted with the price: ${price}`
                    : `تم تقديم عرض جديد بسعر: ${price}`;
                messageType = "update-offer";
              }
            }
            if (type === "accepted-offer") {
              await UpdateStatusOfferSocket(offer, "accepted");
              content =
                user_receiver?.language === "en"
                  ? `Your offer has been accepted. You can now start the service.`
                  : `تم قبول عرضك. يمكنك الآن بدء الخدمة.`;
              messageType = "accepted-offer";
            }
            if (type === "drop") {
              if (typeof DropReasons !== "undefined") {
                content =
                  user_receiver?.language === "en"
                    ? `We're very sorry, but ${user_sender.name} has canceled their offer because: ${DropReasons}`
                    : `نحن آسفون جدًا، ولكن قام ${user_sender.name} بإلغاء عرضه بسبب: ${DropReasons}`;

                await StatusOfferSocket(chat, offer, "drop", DropReasons);
                messageType = "drop";
              }
            }
            if (type === "counter-offer") {
              if (typeof price !== "undefined") {
                content =
                  user_receiver?.language === "en"
                    ? `Can you do it for (${price} AED) ?`
                    : `هل تستطيع فعلها بـ ${price}درهم ؟`;

                messageType = "counter-offer";
              }
            }
            if (type === "canceled") {
              content =
                user_receiver?.language === "en"
                  ? `The offer has been canceled.`
                  : `لقد قام بإلغاء العرض.`;
              await StatusOfferSocket(chat, offer, "canceled");
              messageType = "canceled";
            }
          }
          const conversation = await check_conversation(
            userId,
            receiver,
            service,
            offer
          );
          const conversation_id = conversation._id;
          const chatRoom = `chat_${conversation_id}`;

          console.log(
            "All Connected Sockets:",
            Array.from(chat.sockets.keys())
          );

          if (typeof content !== "undefined") {
            if (!conversation || conversation.status === false) {
              return;
            }
            console.log("send", type);
            await send_message(userId, conversation_id, content, messageType);
          }
          senderSocket.join(chatRoom);
          console.log(`🔵 Sender ${userId} joined ${chatRoom}`);
          const room = chat.adapter.rooms.get(chatRoom);
          console.log("room", room);
          if (room) {
            console.log("room.size", room.size);
            if (room.size == 2) {
              const receiverSocketId = userSocketServices.get(receiver);
              if (receiverSocketId) {
                const receiverSocket = chat.sockets.get(receiverSocketId);
                if (receiverSocket) {
                  const chatDataForReceiver = await GetMessage(
                    conversation_id,
                    receiver,
                    offer
                  );
                  receiverSocket.emit(
                    "Message",
                    JSON.stringify({ Message: chatDataForReceiver })
                  );
                }
              }
            } else if (room.size == 1 && content) {
              await send_notification(
                userId,
                receiver,
                content,
                service,
                "message",
                offer
              );
            }
          }
          const chatData = await GetMessage(conversation_id, userId, offer);
          senderSocket.emit("Message", JSON.stringify({ Message: chatData }));

          const home = await getHomeConversations(receiver);
          chat.to(`user_${receiver}`).emit("Home", JSON.stringify({ home }));
        } else if (event === "home") {
          const home = await getHomeConversations(userId);
          chat.to(`user_${userId}`).emit("Home", JSON.stringify({ home }));
        } else if (event === "leave") {
          const chatRoom = `chat_${conversation_id}`;
          socket.leave(chatRoom);
          console.log(chatRoom, "❌ Client disconnected:", socket.id);
        }
      } catch (err) {
        console.error("❌ Error fetching user categories:", err);
      }
    });

    socket.on("disconnect", () => {
      if (socket.userId) {
        userSockets.delete(socket.userId);
      } // Leave the private room
      socket.leave(`user_${socket.userId}`);
      console.log("❌ Client disconnected:", socket.id);
    });
  });

  const service = io.of("/SocketService");
  service.on("connection", async (socket) => {
    socket.on("MyOffer", async (data) => {
      const { type, userId, status, description, price, serviceId, offerId } =
        data;
      console.log("🔔 [SocketService] Connected:", socket.id);
      if (userId) {
        userSocketServices.set(userId, socket.id);
        socket.join(`user_${userId}`);
        console.log(
          `✅ User ${userId} registered with SocketService ${socket.id} and joined roomm user_${userId}`
        );
        try {
          if (type === "services") {
            const user =
              await UserModel.findById(userId).select("selectedCategories");
            if (user?.selectedCategories?.length) {
              user.selectedCategories.forEach((categoryId) => {
                socket.join(`category_${categoryId}`);
              });
            }
            const services = await NewServiceSocket(
              user.selectedCategories,
              userId
            );
            service.to(socket.id).emit("NewServices", services);
          }
          if (type === "offers") {
            const offers = await GetMyOfferSocket(userId, status);
            service.to(socket.id).emit("Offer", offers);
          }
          if (type === "AddOffer") {
            console.log("description", description);

            const user =
              await UserModel.findById(userId).select("selectedCategories");
            await AddOfferSocket(userId, description, price, serviceId);
            const services = await NewServiceSocket(
              user.selectedCategories,
              userId
            );
            service.to(socket.id).emit("NewServices", services);

            const offers = await GetMyOfferSocket(userId);
            service.to(socket.id).emit("Offer", offers);

            //in screen service
            const Owner = await ServiceModel.findById(serviceId).select("user");

            const ServicesOwner = await GetServicesOfUserocket(Owner.user);
            service.to(`user_${Owner.user}`).emit("MyServices", ServicesOwner);

            const Details = await GetDetails(serviceId);
            service.to(`service_${serviceId}`).emit("Details", Details);
          }
          if (type === "canceled") {
            await StatusOfferSocket(service, offerId, "canceled");
            const offers = await GetMyOfferSocket(userId, status);
            service.to(socket.id).emit("Offer", offers);
          }
        } catch (err) {
          console.error("❌ Error fetching user categories:", err);
        }
      }
    });

    socket.on("MyService", async (data) => {
      const { type, userId, status, serviceId, offerId } = data;
      console.log("🔔 [SocketService] Connected:", socket.id);
      if (userId) {
        console.log("hiiiiiiii22ii");
        userSocketServices.set(userId, socket.id);
        socket.join(`user_${userId}`);
        console.log(
          `✅ User ${userId} registered with SocketService ${socket.id} and joined roomm user_${userId}`
        );
        try {
          if (type === "Service") {
            if (status && Array.isArray(status)) {
              const services = await GetServicesOfUserocket(userId, status);
              service.to(socket.id).emit("MyServices", services);
            } else {
              console.error("❌ Error: status must be an array");
              socket.emit("Error", "status must be an array");
            }
          }
          if (type === "GetDetails") {
            if (serviceId) {
              socket.join(`service_${serviceId}`);
              const Details = await GetDetails(serviceId);
              service.to(socket.id).emit("Details", Details);
            } else {
              console.error("❌ Error: serviceId required");
              socket.emit("Error", "serviceId required");
            }
          }
          if (type === "leaveDetails") {
            if (serviceId) {
              socket.leave(`service_${serviceId}`);
              console.log(
                `❌ User ${userId} removed from leaveDetails and left room ${serviceId}.`
              );
            } else {
              console.error("❌ Error: serviceId required");
              socket.emit("Error", "serviceId required");
            }
          }
          if (type === "OfferStatus") {
            console.error("OfferStatus required");
            if (status === "rejected" || status === "accepted") {
              await StatusOfferSocket(service, offerId, status);
            } else {
              console.error("❌ Error: OfferStatus required");
              socket.emit("Error", "OfferStatus required reject or accepted");
            }
          }
          if (type === "DeleteService") {
            await DeleteService(service, serviceId);
          }
        } catch (err) {
          console.error("❌ Error fetching user categories:", err);
          socket.emit("Error", err);
        }
      }
    });

    socket.on("disconnect", async () => {
      console.log("🔔 [SocketService] Disconnected:", socket.id);

      const userId = socket.userId;
      if (!userId) return;

      try {
        const user =
          await UserModel.findById(userId).select("selectedCategories");

        if (user?.selectedCategories?.length) {
          user.selectedCategories.forEach((categoryId) => {
            socket.leave(`category_${categoryId}`);
          });
        }

        // Clean up user-specific socket and room
        userSocketServices.delete(userId);
        socket.leave(`user_${userId}`);

        console.log(
          `❌ User ${userId} removed from userSocketServices and left all rooms.`
        );
      } catch (error) {
        console.error("Error during socket disconnect cleanup:", error);
      }
    });
  });

  app.set("userSockets", userSockets);
  app.set("userSocketServices", userSocketServices);
};
