const dotenv = require("dotenv");
const ConversationModel = require("../../models/ConversationModel");
const MessageModel = require("../../models/MessageModel");
const OfferModel = require("../../models/OfferModel");
dotenv.config();
exports.check_conversation = async (
  sender_id,
  receiver_id,
  service_id,
  offer_id
) => {
  try {
    // Check if a conversation exists
    let conversation = await ConversationModel.findOne({
      $or: [
        { sender: sender_id, receiver: receiver_id },
        { sender: receiver_id, receiver: sender_id },
      ],
      service: service_id,
      offer: offer_id,
    });

    if (conversation) {
      console.log("✅ Conversation exists:", conversation._id);
      return conversation; // Return existing conversation ID
    }

    // If conversation does not exist, create a new one
    conversation = new ConversationModel({
      sender: sender_id,
      receiver: receiver_id,
      service: service_id,
      offer: offer_id,
      last_message_id: null, // No message yet
      unread_message_count: 0,
    });

    await conversation.save();
    console.log("✅ New conversation created:", conversation._id);
    return conversation;
  } catch (error) {
    console.error("❌ Error in check_conversation:", error);
    throw new Error("Failed to check or create conversation.");
  }
};

exports.send_message = async (sender, conversation_id, content, type) => {
  try {
    const message = new MessageModel({
      conversation_id,
      sender,
      content,
      type,
    });
    await message.save();

    // Update conversation last_message_id and unread_message_count
    await ConversationModel.findByIdAndUpdate(
      conversation_id,
      {
        last_message_id: message._id,
        $inc: { unread_message_count: 1 },
      },
      { new: true }
    );
  } catch (error) {
    console.error("❌ Error in check_conversation:", error);
    throw new Error("Failed to check or create conversation.");
  }
};

exports.GetMessage = async (conversation_id, sender_id, offer_id) => {
  try {
    await MessageModel.updateMany(
      { conversation_id, sender: { $ne: sender_id } },
      { $set: { is_read: true } }
    );

    await ConversationModel.updateOne(
      {
        _id: conversation_id,
        last_message_id: { $ne: sender_id },
      },
      {
        $set: { unread_message_count: 0 },
      }
    );

    const Messages = await MessageModel.find({ conversation_id }).sort({
      createdAt: -1,
    });
    const offer = await OfferModel.findById(offer_id);
    const conversation = await ConversationModel.findById(conversation_id)
      .populate("sender", "name image")
      .populate("receiver", "name image")
      .populate("service", "description images ServiceStatus user");

    let receiver;

    if (conversation.sender._id.toString() === sender_id.toString()) {
      receiver = {
        id: conversation.receiver._id,
        name: conversation.receiver.name,
        image: conversation.receiver.image,
      };
    } else {
      receiver = {
        id: conversation.sender._id,
        name: conversation.sender.name,
        image: conversation.sender.image,
      };
    }

    const response = {
      messages: Messages,
      user: receiver,
      service: conversation.service,
      offer,
      conversation_id: conversation_id,
    };

    return response;
  } catch (error) {
    console.error("GetMessage:", error.message);
    throw error;
  }
};
exports.getHomeConversations = async (senderId) => {
  try {
    const conversations = await ConversationModel.find({
      $or: [{ sender: senderId }, { receiver: senderId }],
    })
      .populate("sender", "name image")
      .populate("receiver", "name image")
      .populate("last_message_id")
      .populate("service", "description images ServiceStatus")
      .populate("offer", "description price OfferStatus")
      .sort({ "last_message_id.createdAt": -1 });
    // Process conversations
    const processedConversations = conversations.map((conversation) => {
      const isSender = String(conversation.sender._id) === String(senderId);
      const user = isSender
        ? {
            id: conversation.receiver._id,
            name: conversation.receiver.name,
            image: conversation.receiver.image,
          }
        : {
            id: conversation.sender._id,
            name: conversation.sender.name,
            image: conversation.sender.image,
          };

      return {
        id: conversation._id,
        last_message: conversation.last_message_id?.content || null,
        last_message_time: conversation.last_message_id?.createdAt || null,
        sender: conversation.last_message_id?.sender || null,
        is_read: conversation.last_message_id?.is_read || null,
        unread_message_count:
          String(conversation.last_send_id) === String(senderId)
            ? 0
            : conversation.unread_message_count,
        user,
        service: conversation.service,
        offer: conversation.offer,
      };
    });

    return processedConversations;
  } catch (error) {
    console.error("Error fetching home conversations:", error);
    throw error;
  }
};
