const jwt = require("jsonwebtoken");
const dotenv = require("dotenv");
const fs = require("fs");
const RatingModel = require("../../models/RatingModel");
const path = require("path");
const { GoogleAuth } = require("google-auth-library");
const axios = require("axios");
const NotifiactionModel = require("../../models/NotifiactionModel");

dotenv.config();
exports.extractPhoneDetails = (phone) => {
  const match = phone.match(/^\+?(\d{1,4})(\d{9,15})$/);
  if (match) {
    return {
      countryCode: match[1],
      mobileNumber: match[2],
    };
  }
  throw new Error("Invalid phone number format");
};

exports.generateToken = (id, phone) => {
  console.log("Generating JWT token for user ID:", id);
  try {
    if (!process.env.JWT_SECRET) throw new Error("JWT_SECRET is missing");
    if (!process.env.JWT_EXPIRATION)
      throw new Error("JWT_EXPIRATION is missing");

    const token = jwt.sign(
      { userID: id, phone: phone },
      process.env.JWT_SECRET,
      { expiresIn: process.env.JWT_EXPIRATION }
    );
    return token;
  } catch (error) {
    throw new Error("Token generation failed: " + error.message);
  }
};

exports.getReview = async (id, status) => {
  try {
    const reviews = await RatingModel.find({
      ratedUser: id,
      status,
    });
    const totalReviews = reviews.length;
    const avgRating =
      totalReviews > 0
        ? reviews.reduce((sum, review) => sum + review.rating, 0) / totalReviews
        : 0;

    return avgRating;
  } catch (error) {
    console.error("Error fetching reviews:", error);
    throw new Error("Failed to fetch reviews: " + error.message);
  }
};
exports.imageClear = async (filePath) => {
  try {
    filePath = path.join(__dirname, "..", "..", filePath);
    fs.unlink(filePath, (err) => console.log(err));
    console.log("Image deleted:", filePath);
  } catch (err) {
    console.log("absolutePath:", filePath);
    console.error("Error deleting image:", err);
  }
};

exports.notification = async (fcm_token, data) => {
  try {
    if (fcm_token) {
      // Path to your Firebase service account JSON file
      const keyFilePath = path.join(__dirname, "..", "..", "firebase.json");

      const auth = new GoogleAuth({
        keyFile: keyFilePath,
        scopes: ["https://www.googleapis.com/auth/firebase.messaging"],
      });

      const client = await auth.getClient();
      const accessTokenResponse = await client.getAccessToken();
      const access_token = accessTokenResponse.token;

      const headers = {
        Authorization: `Bearer ${access_token}`,
        "Content-Type": "application/json",
      };

      const payload = {
        message: {
          token: fcm_token,
          notification: {
            title: data.title,
            body: data.body,
          },
          data: {
            title: data.title,
            body: data.body,
            related_id: data.related_id,
            type: data.type,
            service: data.service,
            offer: data.offer,
          },
        },
      };

      const projectId = "freelance-zone";
      const url = `https://fcm.googleapis.com/v1/projects/${projectId}/messages:send`;

      const response = await axios.post(url, payload, { headers });

      console.log("✅ Notification sent:", response.data);
      console.log("✅ data sent:", data);

    }
    await NotifiactionModel.create({
      data: JSON.stringify(data),
      user: data.receiver,
      type: data.type,
    });
  } catch (err) {
    console.error(
      "❌ Error sending notification:",
      err.response?.data || err.message
    );
  }
};
