const UserModel = require("../../models/UserModel");

class BannerResource {
  constructor(banner, lang) {
    this.banner = banner;
    this.lang = lang;
  }

  async toJSON() {
    try {
      return {
        id: this.banner._id,
        title: this.banner.title,
        default_link: this.banner.default_link,
        image: this.banner[`image_${this.lang}`] || this.banner["image_en"], // Default fallback
        createdAt: this.banner.createdAt,
      };
    } catch (error) {
      console.error("Error fetching user:", error);
      return null; // Return null in case of an error
    }
  }

  static async collection(banners, lang) {
    const promises = banners.map(async banner => {
      const resource = new BannerResource(banner, lang);
      return await resource.toJSON();
    });

    return Promise.all(promises); // Resolve all promises before returning
  }
}

module.exports = BannerResource;
