class BusinessSettingResource {
  constructor(settings) {
    this.settings = settings; // Expecting an array of settings
  }

  async toJSON() {
    try {
      if (!this.settings || !Array.isArray(this.settings)) {
        console.error("settings is undefined or not an array:", this.settings);
        return null;
      }

      const result = {};
      for (const setting of this.settings) {
        result[setting.key] = setting.value;
      }

      return {
        business_name: result["business_name"] || null,
        email: result["email"] || null,
        app_minimum_version_android: result["app_minimum_version_android"]
          ? parseFloat(result["app_minimum_version_android"])
          : null,
        app_minimum_version_ios: result["app_minimum_version_ios"]
          ? parseFloat(result["app_minimum_version_ios"])
          : null,
        currency: result["currency"] || null,
        phone: result["phone"] || null,
        address: result["address"] || null,
        language: result["language"] || null,
        privacy_policy: result["privacy_policy"] || null,
        terms_conditions: result["terms_conditions"] || null,
        web_company: result["web_company"] || null,
        web_app: result["web_app"] || null,
        under_maintenance: String(result["under_maintenance"]).toLowerCase() === 'true',
        company_name: result["company_name"] || null,
      };
    } catch (error) {
      console.error("Error fetching settings:", error);
      return null;
    }
  }

  static async build(settings) {
    const resource = new BusinessSettingResource(settings);
    return await resource.toJSON();
  }
}

module.exports = BusinessSettingResource;
