const UserResource = require("./UserResource");
const OfferResource = require("./OfferResource");

class ServicesResource {
  constructor(service) {
    this.service = service;
  }

  async toJSON() {
    try {
      if (!this.service) {
        console.error("Service is undefined or null:", this.service);
        return null;
      }

      const userResource = this.service.user
        ? await new UserResource(this.service.user, "Provider").toJSON()
        : null;

      // Map over offers using OfferResource.collection()
      const offerResource = this.service.offers
        ? await OfferResource.collection(this.service.offers)
        : [];

      return {
        id: this.service._id,
        description: this.service.description,
        category:this.service.category_id,
        price: this.service.price,
        ServiceStatus: this.service.ServiceStatus,
        userResource,
        location: this.service.locations,
        images: this.service.images || null,
        createdAt: this.service.createdAt,
        offers: offerResource,
      };
    } catch (error) {
      console.error("Error fetching service:", error);
      return null;
    }
  }

  static async collection(services) {
    const promises = services.map(async (service) =>
      new ServicesResource(service).toJSON()
    );
    return Promise.all(promises);
  }
}

module.exports = ServicesResource;
