const { getReview } = require("../logic/helper");
const CategoryResource = require("./CategoryResource");

class UserResource {
  constructor(user, status, lang) {
    this.user = user;
    this.status = status;
    this.lang = lang;
  }

  async toJSON() {
    try {
      if (!this.user) {
        console.error("User is undefined or null:", this.user);
        return null;
      }
      // Await review function since it may be async
      const avgRating = await getReview(this.user._id, this.status);
      const categories = this.user.selectedCategories
        ? this.user.selectedCategories.map(
            (category) => category[`name_${this.lang}`]
          )
        : [];

      return {
        id: this.user._id,
        name: this.user.name,
        image: this.user.image || null,
        avgRating: avgRating,
        categories: this.lang ? categories : [],
        createdAt: this.user.createdAt,
      };
    } catch (error) {
      console.error("Error fetching user:", error);
      return null;
    }
  }

  static async collection(users, status) {
    const promises = users.map(async (user) =>
      new UserResource(user, status).toJSON()
    );
    return Promise.all(promises);
  }
}

module.exports = UserResource;
