const ServicesResource = require("../ServicesResource");

class OfferResource {
  constructor(offer) {
    this.offer = offer;
  }

  async toJSON() {
    try {
      if (!this.offer) {
        console.error("Offer is undefined or null:", this.offer);
        return null;
      }

      const serviceResource = this.offer.service
        ? await new ServicesResource(this.offer.service).toJSON()
        : null;

      return {
        id: this.offer._id,
        description: this.offer.description ?? null,
        price: this.offer.price,
        OfferStatus: this.offer.OfferStatus,
        service: serviceResource,
        createdAt: this.offer.createdAt,
      };
    } catch (error) {
      console.error("Error fetching offer details:", error);
      return null;
    }
  }

  static async collection(offers) {
    const promises = offers.map(async (offer) => {
      const resource = new OfferResource(offer);
      return await resource.toJSON();
    });

    return Promise.all(promises);
  }
}

module.exports = OfferResource;
