const { check } = require("express-validator");
const mongoose = require("mongoose");

const validatorMiddleware = require("../../Middlewares/validatorMiddleware");

exports.GetCategoryValidate = [
  check("id").isMongoId().withMessage("invalid id"),
  validatorMiddleware,
];

exports.AddCategoryValidate = [
  check("name_en")
    .notEmpty()
    .withMessage("name is required in English")
    .isLength({ min: 3 })
    .withMessage("Too short name"),
  check("name_ar")
    .notEmpty()
    .withMessage("name is required in Arabic")
    .isLength({ min: 3 })
    .withMessage("Too short name"),
  check('numberOfLocations')
    .optional()
    .isIn([0, 1, 2])
    .withMessage(' Value should be 0 or 1 or 2 '),
  validatorMiddleware,
];

exports.DeleteCategoryValidate = [
  check("id").isMongoId().withMessage("invalid id"),
  validatorMiddleware,
];

exports.UpdateCategoryValidate = [
  check("id").isMongoId().withMessage("invalid id"),
  validatorMiddleware,
];

exports.InterestesCategoryValidate=[
  check('selectedCategories')
  .isArray().withMessage('selectedCategories should be an array.')
  .custom((value) => {
      // Validate each element in the array is a valid MongoDB ObjectId
      value.forEach((id, index) => {
          if (!mongoose.Types.ObjectId.isValid(id)) {
              throw new Error(`Invalid ObjectId at index ${index}: ${id}`);
          }
      });
      return true;
  }),
  validatorMiddleware,

]
