const { check } = require("express-validator");
const validatorMiddleware = require("../../Middlewares/validatorMiddleware");
const ServiceModel = require("../../models/ServiceModel");
const OfferModel = require("../../models/OfferModel");
const RatingModel = require("../../models/RatingModel");

exports.AddReviewValidate = [
  // Validate serviceId
  check("serviceId")
    .isMongoId()
    .withMessage("Invalid service ID")
    .custom(async (serviceId, { req }) => {
      const service = await ServiceModel.findOne({
        _id: serviceId,
      });

      if (!service) {
        throw new Error(`Service not found`);
      }

      req.service = service;
      return true;
    }),

  // Validate offerId
  check("offerId")
    .isMongoId()
    .withMessage("Invalid Offer ID")
    .custom(async (offerId, { req }) => {
      const offer = await OfferModel.findOne({
        _id: offerId,
      });

      if (!offer) {
        throw new Error(`Service not found `);
      }
      req.offer = offer;
      return true;
    }),
  // Validate ratedUser
  check("ratedUser")
    .isMongoId()
    .withMessage("Invalid ratedUser ID")
    .custom(async (ratedUserId, { req }) => {
      const service = req.service;
      const offer = req.offer;
      const currentUser = req.user._id.toString(); // Current user making the rating

      if (!service || !offer) {
        throw new Error(
          `Service or offer not found. Ensure valid IDs .${existingOffer}`
        );
      }

      // Determine valid ratedUser
      const serviceOwner = service.user.toString();
      const offerOwner = offer.user.toString();

      if (![serviceOwner, offerOwner].includes(ratedUserId)) {
        throw new Error("You can only rate the service owner or offer owner.");
      }

      // Ensure user is not rating themselves
      if (currentUser === ratedUserId) {
        throw new Error("You cannot rate yourself.");
      }

      // Ensure user hasn't rated this transaction before
      const existingRating = await RatingModel.findOne({
        service: service._id,
        offer: offer._id,
        ratedBy: currentUser,
      });

      if (existingRating) {
        throw new Error("You have already rated this user.");
      }

      // Set req.status based on who is rating whom
      if (currentUser === serviceOwner) {
        req.status = "Implemented";
      } else if (currentUser === offerOwner) {
        req.status = "Provider";
      }

      return true;
    }),

  // Validate rating (1-5)
  check("rating")
    .isInt({ min: 1, max: 5 })
    .withMessage("Rating must be a number between 1 and 5"),

  // Validate review (string)
  check("review")
    .isString()
    .notEmpty()
    .withMessage("Review is required and must be a string"),

  // Apply middleware to return errors
  validatorMiddleware,
];

exports.GetReviewValidate = [
  check("id").isMongoId().withMessage("invalid userId"),
  check("status")
    .isIn(["Provider", "Implemented"])
    .withMessage("Invalid status value."),
  validatorMiddleware,
];
